@@ -1,5 +1,5 @@
 /* **************************************************************************
- * Copyright (C) 2016-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2016-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -125,6 +125,13 @@ try
             "                           Reported time will be the average.\n"
             "                           ")
 
+        ("alg_mode",
+         value<rocblas_int>(&argus.alg_mode)->default_value(0),
+            "0 = GPU-only, 1 = Hybrid\n"
+            "                           This will change how the algorithm operates.\n"
+            "                           Only applicable to functions with hybrid support.\n"
+            "                           ")
+
         ("mem_query",
          value<rocblas_int>(&argus.mem_query)->default_value(0),
             "Calculate the required amount of device workspace memory? 0 = No, 1 = Yes.\n"
@@ -169,6 +176,12 @@ try
             "                           This will additionally print the relative error of the computations.\n"
             "                           ")
 
+        ("hash",
+         value<rocblas_int>(&argus.hash_check)->default_value(0),
+            "Print hash of GPU results? 0 = No, 1 = Yes.\n"
+            "                           Meant for checking reproducibility of computations.\n"
+            "                           ")
+
         // size options
         ("k",
          value<rocblas_int>(),
@@ -565,6 +578,12 @@ try
             "                           The order in which a series of transformations are applied.\n"
             "                           ")
 
+        ("pivot",
+         value<char>()->default_value('V'),
+            "V = variable, T = top, B = bottom.\n"
+            "                           Defines the planes on which a sequence of rotations is applied.\n"
+            "                           ")
+
         ("evect",
          value<char>()->default_value('N'),
             "N = none, V = compute eigenvectors of the matrix,\n"
@@ -602,6 +621,18 @@ try
             "                           Indicates if a matrix should be transposed.\n"
             "                           ")
 
+        ("transA",
+         value<char>()->default_value('N'),
+            "N = no transpose, T = transpose, C = conjugate transpose.\n"
+            "                           Indicates if matrix A should be transposed.\n"
+            "                           ")
+
+        ("transB",
+         value<char>()->default_value('N'),
+            "N = no transpose, T = transpose, C = conjugate transpose.\n"
+            "                           Indicates if matrix B should be transposed.\n"
+            "                           ")
+
         ("uplo",
          value<char>()->default_value('U'),
             "U = upper, L = lower.\n"
@@ -644,6 +675,7 @@ try
     argus.validate_fill("uplo");
     argus.validate_diag("diag");
     argus.validate_direct("direct");
+    argus.validate_pivot("pivot");
     argus.validate_storev("storev");
     argus.validate_svect("svect");
     argus.validate_svect("left_svect");
