@@ -1,5 +1,5 @@
 /* **************************************************************************
- * Copyright (C) 2021-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,6 +42,10 @@
 #include "fmt_rocblas_types.hpp"
 #include "rocblas_utility.hpp"
 
+#ifdef ROCSOLVER_LIBRARY
+ROCSOLVER_BEGIN_NAMESPACE
+#endif
+
 /*
  * ===========================================================================
  *    common location for functions that are used by both the rocSOLVER
@@ -60,6 +64,7 @@
     } while(0)
 
 /* =============================================================================================== */
+
 /* Number properties functions.                                                                    */
 
 template <typename T>
@@ -96,6 +101,12 @@ double get_time_us_sync(hipStream_t stream);
  * time */
 double get_time_us_no_sync();
 
+/* =============================================================================================== */
+/* Device query functions.                                                                         */
+
+/*! \brief Get warp size of the current device */
+int get_device_warp_size();
+
 /* =============================================================================================== */
 /* Print functions.                                                                                */
 inline void pairs_to_string(std::string& str, const char* sep)
@@ -152,11 +163,11 @@ void print_to_stream(std::ostream& os,
     else
     {
         // symmetric case
-        for(int i = 0; i < min(m, n); i++)
+        for(int i = 0; i < std::min(m, n); i++)
         {
             if(!empty)
                 s += "    ";
-            for(int j = 0; j < min(m, n); j++)
+            for(int j = 0; j < std::min(m, n); j++)
             {
                 if(uplo == rocblas_fill_upper)
                 {
@@ -198,7 +209,7 @@ void print_device_matrix(std::ostream& os,
                          const rocblas_fill uplo = rocblas_fill_full,
                          const rocblas_int inca = 1)
 {
-    size_t to_read = max(inca * (m - 1) + m, lda * (n - 1) + n);
+    size_t to_read = std::max(inca * (m - 1) + n, lda * (n - 1) + m);
 
     std::vector<T> hA(to_read);
     THROW_IF_HIP_ERROR(
@@ -220,7 +231,7 @@ void print_device_matrix(std::ostream& os,
                          const rocblas_fill uplo = rocblas_fill_full,
                          const rocblas_int inca = 1)
 {
-    size_t to_read = max(inca * (m - 1) + m, lda * (n - 1) + n);
+    size_t to_read = std::max(inca * (m - 1) + n, lda * (n - 1) + m);
 
     std::vector<T> hA(to_read);
     T* AA[1];
@@ -242,7 +253,7 @@ void print_device_matrix(const std::string file,
                          const rocblas_fill uplo = rocblas_fill_full,
                          const rocblas_int inca = 1)
 {
-    size_t to_read = max(inca * (m - 1) + m, lda * (n - 1) + n);
+    size_t to_read = std::max(inca * (m - 1) + n, lda * (n - 1) + m);
 
     std::ofstream os(file);
     std::vector<T> hA(to_read);
@@ -264,7 +275,7 @@ void print_device_matrix(const std::string file,
                          const rocblas_fill uplo = rocblas_fill_full,
                          const rocblas_int inca = 1)
 {
-    size_t to_read = max(inca * (m - 1) + m, lda * (n - 1) + n);
+    size_t to_read = std::max(inca * (m - 1) + n, lda * (n - 1) + m);
 
     std::ofstream os(file);
     std::vector<T> hA(to_read);
@@ -550,3 +561,7 @@ inline void read_matrix(const std::string filenameS,
             fmt::format("Error: Could not close file {} with test data...", filename));
     }
 }
+
+#ifdef ROCSOLVER_LIBRARY
+ROCSOLVER_END_NAMESPACE
+#endif
