@@ -4,7 +4,7 @@
  *     Univ. of Tennessee, Univ. of California Berkeley,
  *     Univ. of Colorado Denver and NAG Ltd..
  *     June 2017
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,6 +38,8 @@
 
 #include <cmath>
 
+#include "rocauxiliary_bdsqr_hybrid.hpp"
+
 ROCSOLVER_BEGIN_NAMESPACE
 
 /******************** Device functions *************************/
@@ -64,7 +66,10 @@ __device__ T bdsqr_estimate(const rocblas_int n, T* D, T* E, int t2b, T tol, int
             smin = -1;
             break;
         }
-        t = std::abs(D[jd]) * t / (t + std::abs(E[je]));
+
+        // Note: Order of operations is important to prevent t from being rounded down to zero
+        t = std::abs(D[jd]) * (t / (t + std::abs(E[je])));
+
         smin = (t < smin) ? t : smin;
     }
 
@@ -83,10 +88,13 @@ __device__ void bdsqr_QRstep(const rocblas_int tid,
                              S* D,
                              S* E,
                              T* V,
+                             const rocblas_stride shiftV,
                              const rocblas_int ldv,
                              T* U,
+                             const rocblas_stride shiftU,
                              const rocblas_int ldu,
                              T* C,
+                             const rocblas_stride shiftC,
                              const rocblas_int ldc,
                              const S sh,
                              S* rots)
@@ -168,6 +176,8 @@ __device__ void bdsqr_QRstep(const rocblas_int tid,
     // update singular vectors
     if(V && nv)
     {
+        V += shiftV;
+
         // rotate from the left
         for(rocblas_int j = tid; j < nv; j += hipBlockDim_x)
         {
@@ -181,15 +191,18 @@ __device__ void bdsqr_QRstep(const rocblas_int tid,
                 c = rots[rk];
                 s = rots[rk + n];
                 V[k + j * ldv] = c * temp1 - s * temp2;
-                V[(k + dir) + j * ldv] = temp1 = c * temp2 + s * temp1;
+                temp1 = c * temp2 + s * temp1;
 
                 k += dir;
                 rk += dir;
             }
+            V[k + j * ldv] = temp1;
         }
     }
     if(U && nu)
     {
+        U += shiftU;
+
         // rotate from the right
         for(rocblas_int i = tid; i < nu; i += hipBlockDim_x)
         {
@@ -203,15 +216,18 @@ __device__ void bdsqr_QRstep(const rocblas_int tid,
                 c = rots[rk];
                 s = rots[rk + n];
                 U[i + k * ldu] = c * temp1 - s * temp2;
-                U[i + (k + dir) * ldu] = temp1 = c * temp2 + s * temp1;
+                temp1 = c * temp2 + s * temp1;
 
                 k += dir;
                 rk += dir;
             }
+            U[i + k * ldu] = temp1;
         }
     }
     if(C && nc)
     {
+        C += shiftC;
+
         // rotate from the left
         for(rocblas_int j = tid; j < nc; j += hipBlockDim_x)
         {
@@ -225,11 +241,12 @@ __device__ void bdsqr_QRstep(const rocblas_int tid,
                 c = rots[rk];
                 s = rots[rk + n];
                 C[k + j * ldc] = c * temp1 - s * temp2;
-                C[(k + dir) + j * ldc] = temp1 = c * temp2 + s * temp1;
+                temp1 = c * temp2 + s * temp1;
 
                 k += dir;
                 rk += dir;
             }
+            C[k + j * ldc] = temp1;
         }
     }
 }
@@ -525,8 +542,9 @@ ROCSOLVER_KERNEL void bdsqr_lower2upper(const rocblas_int n,
                 c = rots[j];
                 s = rots[j + n];
                 U[i + j * ldu] = c * temp1 - s * temp2;
-                U[i + (j + 1) * ldu] = temp1 = c * temp2 + s * temp1;
+                temp1 = c * temp2 + s * temp1;
             }
+            U[i + (n - 1) * ldu] = temp1;
         }
     }
     if(nc)
@@ -541,8 +559,9 @@ ROCSOLVER_KERNEL void bdsqr_lower2upper(const rocblas_int n,
                 c = rots[i];
                 s = rots[i + n];
                 C[i + j * ldc] = c * temp1 - s * temp2;
-                C[(i + 1) + j * ldc] = temp1 = c * temp2 + s * temp1;
+                temp1 = c * temp2 + s * temp1;
             }
+            C[(n - 1) + j * ldc] = temp1;
         }
     }
 }
@@ -667,8 +686,8 @@ ROCSOLVER_KERNEL void bdsqr_compute(const rocblas_int n,
             if(tid == 0)
                 splits[4 * sid] = (t2b ? 1 : -1);
 
-            bdsqr_QRstep(tid, t2b, k - i + 1, nv, nu, nc, D + i, E + i, V + i, ldv, U + i * ldu,
-                         ldu, C + i, ldc, smin, rots + incW * i);
+            bdsqr_QRstep(tid, t2b, k - i + 1, nv, nu, nc, D + i, E + i, V, i, ldv, U, i * ldu, ldu,
+                         C, i, ldc, smin, rots + incW * i);
         }
         else
         {
@@ -767,11 +786,12 @@ ROCSOLVER_KERNEL void bdsqr_rotate(const rocblas_int n,
                     c = rots[rk];
                     s = rots[rk + nn];
                     V[k + tid * ldv] = c * temp1 - s * temp2;
-                    V[(k + dir) + tid * ldv] = temp1 = c * temp2 + s * temp1;
+                    temp1 = c * temp2 + s * temp1;
 
                     k += dir;
                     rk += dir;
                 }
+                V[k + tid * ldv] = temp1;
             }
             if(U && tid < nu)
             {
@@ -786,11 +806,12 @@ ROCSOLVER_KERNEL void bdsqr_rotate(const rocblas_int n,
                     c = rots[rk];
                     s = rots[rk + nn];
                     U[tid + k * ldu] = c * temp1 - s * temp2;
-                    U[tid + (k + dir) * ldu] = temp1 = c * temp2 + s * temp1;
+                    temp1 = c * temp2 + s * temp1;
 
                     k += dir;
                     rk += dir;
                 }
+                U[tid + k * ldu] = temp1;
             }
             if(C && tid < nc)
             {
@@ -805,11 +826,12 @@ ROCSOLVER_KERNEL void bdsqr_rotate(const rocblas_int n,
                     c = rots[rk];
                     s = rots[rk + nn];
                     C[k + tid * ldc] = c * temp1 - s * temp2;
-                    C[(k + dir) + tid * ldc] = temp1 = c * temp2 + s * temp1;
+                    temp1 = c * temp2 + s * temp1;
 
                     k += dir;
                     rk += dir;
                 }
+                C[k + tid * ldc] = temp1;
             }
         }
     }
@@ -1204,6 +1226,9 @@ rocblas_status rocsolver_bdsqr_template(rocblas_handle handle,
     hipStream_t stream;
     rocblas_get_stream(handle, &stream);
 
+    rocsolver_alg_mode alg_mode;
+    ROCBLAS_CHECK(rocsolver_get_alg_mode(handle, rocsolver_function_bdsqr, &alg_mode));
+
     // set tolerance and max number of iterations:
     // machine precision (considering rounding strategy)
     S eps = get_epsilon<S>() / 2;
@@ -1248,70 +1273,81 @@ rocblas_status rocsolver_bdsqr_template(rocblas_handle handle,
 
     if(n > 1)
     {
-        // rotate to upper bidiagonal if necessary
-        if(uplo == rocblas_fill_lower)
+        if(alg_mode == rocsolver_alg_mode_hybrid)
         {
-            ROCSOLVER_LAUNCH_KERNEL((bdsqr_lower2upper<T>), gridBasic, threadsUC, 0, stream, n, nu,
-                                    nc, D, strideD, E, strideE, U, shiftU, ldu, strideU, C, shiftC,
-                                    ldc, strideC, info, work, strideW, completed);
+            ROCBLAS_CHECK(rocsolver_bdsqr_host_batch_template<T, S, W1, W2, W3, rocblas_int>(
+                handle, uplo, n, nv, nu, nc, D, strideD, E, strideE, V, shiftV, ldv, strideV, U,
+                shiftU, ldu, strideU, C, shiftC, ldc, strideC, info, batch_count, splits_map, work));
         }
-
-        rocblas_int h_iter = 0;
-        struct
+        else
         {
-            rocblas_int completed;
-            rocblas_int num_splits;
-        } h_params;
-
-        while(h_iter < maxiter)
-        {
-            // if all instances in the batch have finished, exit the loop
-            HIP_CHECK(hipMemcpyAsync(&h_params, completed, sizeof(h_params), hipMemcpyDeviceToHost,
-                                     stream));
-            HIP_CHECK(hipStreamSynchronize(stream));
-
-            if(h_params.completed == batch_count)
-                break;
-
-            dim3 gridSplits(1, h_params.num_splits, batch_count);
-            dim3 gridVUC((nvuc_max - 1) / BS1 + 1, h_params.num_splits, batch_count);
-
-            for(rocblas_int inner_iters = 0; inner_iters < BDSQR_ITERS_PER_SYNC; inner_iters++)
+            // rotate to upper bidiagonal if necessary
+            if(uplo == rocblas_fill_lower)
             {
-                if(nvuc_max <= BDSQR_SWITCH_SIZE)
-                {
-                    // main computation of SVD
-                    ROCSOLVER_LAUNCH_KERNEL((bdsqr_compute<BS1, T>), gridSplits, threadsBS1, 0,
-                                            stream, n, nv, nu, nc, D, strideD, E, strideE, V,
-                                            shiftV, ldv, strideV, U, shiftU, ldu, strideU, C,
-                                            shiftC, ldc, strideC, maxiter, eps, sfm, tol, minshift,
-                                            splits_map, work, incW, strideW, completed);
-                }
-                else
-                {
-                    // main computation of SVD
-                    ROCSOLVER_LAUNCH_KERNEL(
-                        (bdsqr_compute<BS1, T>), gridSplits, threadsBS1, 0, stream, n, nv, nu, nc,
-                        D, strideD, E, strideE, (W1) nullptr, shiftV, ldv, strideV, (W2) nullptr,
-                        shiftU, ldu, strideU, (W3) nullptr, shiftC, ldc, strideC, maxiter, eps, sfm,
-                        tol, minshift, splits_map, work, incW, strideW, completed);
-
-                    // update singular vectors
-                    ROCSOLVER_LAUNCH_KERNEL((bdsqr_rotate<T>), gridVUC, threadsVUC, 0, stream, n,
-                                            nv, nu, nc, V, shiftV, ldv, strideV, U, shiftU, ldu,
-                                            strideU, C, shiftC, ldc, strideC, maxiter, splits_map,
-                                            work, incW, strideW, completed);
-                }
-
-                // update split block endpoints
-                ROCSOLVER_LAUNCH_KERNEL((bdsqr_update_endpoints<T>), gridSplits, threadsBasic, 0,
-                                        stream, n, E, strideE, splits_map, work, strideW, completed);
+                ROCSOLVER_LAUNCH_KERNEL((bdsqr_lower2upper<T>), gridBasic, threadsUC, 0, stream, n,
+                                        nu, nc, D, strideD, E, strideE, U, shiftU, ldu, strideU, C,
+                                        shiftC, ldc, strideC, info, work, strideW, completed);
             }
 
-            // check for completion
-            h_iter += BDSQR_ITERS_PER_SYNC;
-            ROCSOLVER_LAUNCH_KERNEL((bdsqr_chk_completed<T>), gridBasic, threadsBasic, 0, stream, n,
-                                    maxiter, splits_map, work, strideW, completed);
+            rocblas_int h_iter = 0;
+            struct
+            {
+                rocblas_int completed;
+                rocblas_int num_splits;
+            } h_params;
+
+            while(h_iter < maxiter)
+            {
+                // if all instances in the batch have finished, exit the loop
+                HIP_CHECK(hipMemcpyAsync(&h_params, completed, sizeof(h_params),
+                                         hipMemcpyDeviceToHost, stream));
+                HIP_CHECK(hipStreamSynchronize(stream));
+
+                if(h_params.completed == batch_count)
+                    break;
+
+                dim3 gridSplits(1, h_params.num_splits, batch_count);
+                dim3 gridVUC((nvuc_max - 1) / BS1 + 1, h_params.num_splits, batch_count);
+
+                for(rocblas_int inner_iters = 0; inner_iters < BDSQR_ITERS_PER_SYNC; inner_iters++)
+                {
+                    if(nvuc_max <= BDSQR_SWITCH_SIZE)
+                    {
+                        // main computation of SVD
+                        ROCSOLVER_LAUNCH_KERNEL((bdsqr_compute<BS1, T>), gridSplits, threadsBS1, 0,
+                                                stream, n, nv, nu, nc, D, strideD, E, strideE, V,
+                                                shiftV, ldv, strideV, U, shiftU, ldu, strideU, C,
+                                                shiftC, ldc, strideC, maxiter, eps, sfm, tol,
+                                                minshift, splits_map, work, incW, strideW, completed);
+                    }
+                    else
+                    {
+                        // main computation of SVD
+                        ROCSOLVER_LAUNCH_KERNEL((bdsqr_compute<BS1, T>), gridSplits, threadsBS1, 0,
+                                                stream, n, nv, nu, nc, D, strideD, E, strideE,
+                                                (W1) nullptr, shiftV, ldv, strideV, (W2) nullptr,
+                                                shiftU, ldu, strideU, (W3) nullptr, shiftC, ldc,
+                                                strideC, maxiter, eps, sfm, tol, minshift,
+                                                splits_map, work, incW, strideW, completed);
+
+                        // update singular vectors
+                        ROCSOLVER_LAUNCH_KERNEL((bdsqr_rotate<T>), gridVUC, threadsVUC, 0, stream,
+                                                n, nv, nu, nc, V, shiftV, ldv, strideV, U, shiftU,
+                                                ldu, strideU, C, shiftC, ldc, strideC, maxiter,
+                                                splits_map, work, incW, strideW, completed);
+                    }
+
+                    // update split block endpoints
+                    ROCSOLVER_LAUNCH_KERNEL((bdsqr_update_endpoints<T>), gridSplits, threadsBasic,
+                                            0, stream, n, E, strideE, splits_map, work, strideW,
+                                            completed);
+                }
+
+                // check for completion
+                h_iter += BDSQR_ITERS_PER_SYNC;
+                ROCSOLVER_LAUNCH_KERNEL((bdsqr_chk_completed<T>), gridBasic, threadsBasic, 0,
+                                        stream, n, maxiter, splits_map, work, strideW, completed);
+            }
         }
     }
 
