@@ -1,5 +1,5 @@
 /* **************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,7 +27,7 @@
 
 #pragma once
 
-#include "rocblas.hpp"
+#include "rocblas/rocblas.h"
 #include "rocsolver/rocsolver.h"
 
 ROCSOLVER_BEGIN_NAMESPACE
@@ -59,13 +59,13 @@ rocblas_status larf_run_small(rocblas_handle handle,
                               const I batch_count);
 
 // larfg
-template <typename T, typename I, typename U>
+template <typename T, typename I, typename S, typename U>
 rocblas_status larfg_run_small(rocblas_handle handle,
                                const I n,
                                U alpha,
                                const rocblas_stride shiftA,
                                const rocblas_stride strideA,
-                               T* beta,
+                               S* beta,
                                const rocblas_stride shiftB,
                                const rocblas_stride strideB,
                                U x,
@@ -188,55 +188,55 @@ rocblas_status rocsolver_trsm_upper(rocblas_handle handle,
 
 // gemm
 template <typename T, typename I, typename U1, typename U2, typename U3>
-rocblas_status rocsolver_gemm(rocblas_handle handle,
-                              rocblas_operation transA,
-                              rocblas_operation transB,
-                              I m,
-                              I n,
-                              I k,
-                              const T* alpha,
-                              U1 A,
-                              rocblas_stride shiftA,
-                              I lda,
-                              rocblas_stride strideA,
-                              U2 B,
-                              rocblas_stride shiftB,
-                              I ldb,
-                              rocblas_stride strideB,
-                              const T* beta,
-                              U3 C,
-                              rocblas_stride shiftC,
-                              I ldc,
-                              rocblas_stride strideC,
-                              I batch_count,
-                              T** work);
+ROCSOLVER_EXPORT rocblas_status rocsolver_gemm(rocblas_handle handle,
+                                               rocblas_operation transA,
+                                               rocblas_operation transB,
+                                               I m,
+                                               I n,
+                                               I k,
+                                               const T* alpha,
+                                               U1 A,
+                                               rocblas_stride shiftA,
+                                               I lda,
+                                               rocblas_stride strideA,
+                                               U2 B,
+                                               rocblas_stride shiftB,
+                                               I ldb,
+                                               rocblas_stride strideB,
+                                               const T* beta,
+                                               U3 C,
+                                               rocblas_stride shiftC,
+                                               I ldc,
+                                               rocblas_stride strideC,
+                                               I batch_count,
+                                               T** work);
 
 template <typename T, typename I, typename U1, typename U2, typename U3>
-rocblas_status rocsolver_gemm(rocblas_handle handle,
-                              rocblas_operation transA,
-                              rocblas_operation transB,
-                              I m,
-                              I n,
-                              I k,
-                              const T* alpha,
-                              U1 A,
-                              rocblas_stride shiftA,
-                              I inca,
-                              I lda,
-                              rocblas_stride strideA,
-                              U2 B,
-                              rocblas_stride shiftB,
-                              I incb,
-                              I ldb,
-                              rocblas_stride strideB,
-                              const T* beta,
-                              U3 C,
-                              rocblas_stride shiftC,
-                              I incc,
-                              I ldc,
-                              rocblas_stride strideC,
-                              I batch_count,
-                              T** work);
+ROCSOLVER_EXPORT rocblas_status rocsolver_gemm(rocblas_handle handle,
+                                               rocblas_operation transA,
+                                               rocblas_operation transB,
+                                               I m,
+                                               I n,
+                                               I k,
+                                               const T* alpha,
+                                               U1 A,
+                                               rocblas_stride shiftA,
+                                               I inca,
+                                               I lda,
+                                               rocblas_stride strideA,
+                                               U2 B,
+                                               rocblas_stride shiftB,
+                                               I incb,
+                                               I ldb,
+                                               rocblas_stride strideB,
+                                               const T* beta,
+                                               U3 C,
+                                               rocblas_stride shiftC,
+                                               I incc,
+                                               I ldc,
+                                               rocblas_stride strideC,
+                                               I batch_count,
+                                               T** work);
 
 // ger
 template <bool CONJ, typename T, typename I, typename U>
