@@ -1,5 +1,5 @@
 /* **************************************************************************
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,6 +26,9 @@
  * *************************************************************************/
 
 #include "roclapack_gesvdx_notransv.hpp"
+
+ROCSOLVER_BEGIN_NAMESPACE
+
 /*
  * ===========================================================================
  *    gesvdx_notransv is not intended for inclusion in the public API. It
@@ -187,6 +190,8 @@ rocblas_status rocsolver_gesvdx_notransv_strided_batched_impl(rocblas_handle han
         (TT*)tmpDE, (T*)tauqp, (TT*)tmpZ, (T*)tau, (T*)tmpT, (T**)workArr, (T**)size_workArr2);
 }
 
+ROCSOLVER_END_NAMESPACE
+
 /*
  * ===========================================================================
  *    C wrapper
@@ -223,7 +228,7 @@ ROCSOLVER_EXPORT rocblas_status
                                                rocblas_int* info,
                                                const rocblas_int batch_count)
 {
-    return rocsolver_gesvdx_notransv_strided_batched_impl<float>(
+    return rocsolver::rocsolver_gesvdx_notransv_strided_batched_impl<float>(
         handle, left_svect, right_svect, srange, m, n, A, lda, strideA, vl, vu, il, iu, nsv, S,
         strideS, U, ldu, strideU, V, ldv, strideV, ifail, strideF, info, batch_count);
 }
@@ -256,7 +261,7 @@ ROCSOLVER_EXPORT rocblas_status
                                                rocblas_int* info,
                                                const rocblas_int batch_count)
 {
-    return rocsolver_gesvdx_notransv_strided_batched_impl<double>(
+    return rocsolver::rocsolver_gesvdx_notransv_strided_batched_impl<double>(
         handle, left_svect, right_svect, srange, m, n, A, lda, strideA, vl, vu, il, iu, nsv, S,
         strideS, U, ldu, strideU, V, ldv, strideV, ifail, strideF, info, batch_count);
 }
@@ -289,7 +294,7 @@ ROCSOLVER_EXPORT rocblas_status
                                                rocblas_int* info,
                                                const rocblas_int batch_count)
 {
-    return rocsolver_gesvdx_notransv_strided_batched_impl<rocblas_float_complex>(
+    return rocsolver::rocsolver_gesvdx_notransv_strided_batched_impl<rocblas_float_complex>(
         handle, left_svect, right_svect, srange, m, n, A, lda, strideA, vl, vu, il, iu, nsv, S,
         strideS, U, ldu, strideU, V, ldv, strideV, ifail, strideF, info, batch_count);
 }
@@ -322,7 +327,7 @@ ROCSOLVER_EXPORT rocblas_status
                                                rocblas_int* info,
                                                const rocblas_int batch_count)
 {
-    return rocsolver_gesvdx_notransv_strided_batched_impl<rocblas_double_complex>(
+    return rocsolver::rocsolver_gesvdx_notransv_strided_batched_impl<rocblas_double_complex>(
         handle, left_svect, right_svect, srange, m, n, A, lda, strideA, vl, vu, il, iu, nsv, S,
         strideS, U, ldu, strideU, V, ldv, strideV, ifail, strideF, info, batch_count);
 }
