@@ -1,5 +1,5 @@
 /* **************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -79,9 +79,9 @@ rocblas_status rocsolver_sygvd_hegvd_strided_batched_impl(rocblas_handle handle,
     // size of temporary info array
     size_t size_iinfo;
     rocsolver_sygvd_hegvd_getMemorySize<false, true, T, S>(
-        itype, evect, uplo, n, batch_count, &size_scalars, &size_work1, &size_work2, &size_work3,
-        &size_work4, &size_tmpz, &size_splits, &size_tau, &size_pivots_workArr, &size_iinfo,
-        &optim_mem);
+        handle, itype, evect, uplo, n, batch_count, &size_scalars, &size_work1, &size_work2,
+        &size_work3, &size_work4, &size_tmpz, &size_splits, &size_tau, &size_pivots_workArr,
+        &size_iinfo, &optim_mem);
 
     if(rocblas_is_device_memory_size_query(handle))
         return rocblas_set_optimal_device_memory_size(handle, size_scalars, size_work1, size_work2,
